/////////////////////////////////////////////////////////////////////
// File    : Statlogging.cpp
// Desc    : 
// Created : Friday, May 25, 2001
// Author  : Shelby
// 
// (c) 2001 Relic Entertainment Inc.
//

#include "pch.h"
#include "Stats/StatLogging.h"

#include "ModObj.h"
#include "SigmaWorld.h"

#include <EngineAPI/EntityFactory.h>

#include <SimEngine/LuaSimEngineTypes.h>
#include <SimEngine/Entity.h>

#include <LuaConfig/LuaConfig.h>
#include <LuaConfig/LuaLibMan.h>

///////////////////////////////////////////////////////////////////// 
// add extensions to luabinding here

namespace LuaBinding
{
	///////////////////////////////////////////////////////////////////// 
	// ControllerBlueprint Pointer Custom Paramater

	template<> inline bool ParmTraits<const ControllerBlueprint*>::Pop( type& t, lua_State* ls, int n )
	{
		int entitytag = LuaConfig::GetTag( ls, "entity" );
		int cbptag = LuaConfig::GetTag( ls, "controllerBP" );

		if (lua_type( ls, n) == LUA_TNUMBER)
		{
			int ebpNetID = static_cast<int>(lua_tonumber( ls, n ));
			t = ModObj::i()->GetWorld()->GetEntityFactory()->GetControllerBP( (const EBPNetworkID)ebpNetID );
		}
		else
		if (cbptag == lua_tag( ls, n ))
		{
			t = static_cast<const ControllerBlueprint*>(lua_touserdata( ls, n ));
		}
		else
		if (entitytag == lua_tag( ls, n ))
		{
			Entity* entity = static_cast<Entity*>(lua_touserdata( ls, n ));
			if (entity)
			{
				t = entity->GetControllerBP();
			}
		}
		else
		{
			return false;
		}

		return true;
	}

	template<> inline void ParmTraits<const ControllerBlueprint*>::Push( const type t, lua_State* ls )
	{
		int cbptag = LuaConfig::GetTag( ls, "controllerBP" );
		
		lua_pushusertag( ls, (void*)t, cbptag);

		return;
	}

}

///////////////////////////////////////////////////////////////////// 
// 

extern const char* LUALIB_STATLOGGING = "statlogging";

namespace
{
	class LuaStatLoggingLib : public LuaLibrary
	{
	public:
		LuaStatLoggingLib( )
		{
			LuaLibMan::AddLibrary( LUALIB_STATLOGGING, this );
		}

		virtual ~LuaStatLoggingLib( )
		{
			LuaLibMan::RemLibrary( LUALIB_STATLOGGING, this );
		}

		virtual bool Register( LuaConfig& lc )
		{
		#define BINDFUNC(f) \
			StoreLuaBindingObj( lc, LuaBinding::Bind( &lc, "Stats_"#f, StatLogging::Instance(), &StatLogging::f ) );
	
			BINDFUNC( KillsTotal				);
			BINDFUNC( LossesTotal				);
			BINDFUNC( CreaturesBuilt			);
			BINDFUNC( CreaturesLost				);
			BINDFUNC( HenchmenBuilt				);
			BINDFUNC( HenchmenLost				);
			BINDFUNC( HenchmenMax				);
			BINDFUNC( BuildingsKilled			);
			BINDFUNC( BuildingsBuilt			);
			BINDFUNC( BuildingsLost				);
			BINDFUNC( BuildingsActive			);
			BINDFUNC( MilitaryUnitsActive		);
			BINDFUNC( BuildingsTotalScrap		);
			BINDFUNC( BuildingsTotalElectricity	);
			BINDFUNC( AddOnsTotalScrap			);
			BINDFUNC( AddOnsTotaElectricity		);
			BINDFUNC( MapExplored				);
			BINDFUNC( ScrapGathered				);
			BINDFUNC( ScrapSpent				);			
			BINDFUNC( ElectricityGathered		);	
			BINDFUNC( ElectricitySpent			);
			BINDFUNC( TechResearched			);
			BINDFUNC( TechResearchedCount		);
			BINDFUNC( TechTotalScrap			);
			BINDFUNC( TechTotalElectricity		);
			BINDFUNC( CreaturesTypeCount		);
			BINDFUNC( CreaturesTypeAt			);
			BINDFUNC( CreatureCostValue			);
			BINDFUNC( CreatureStructureDmgValue );
			BINDFUNC( CreatureCreatureDmgValue  );
			BINDFUNC( CreatureStructureDmg		);
			BINDFUNC( CreatureCreatureDmg		);
			BINDFUNC( CreatureRank				);
			BINDFUNC( CreatureEffectiveness     );
			BINDFUNC( CreatureMax				);
			BINDFUNC( PlayerCount				);
			BINDFUNC( PlayerAt					);
			BINDFUNC( RankFinal				    );
			BINDFUNC( Rank2Time					);
			BINDFUNC( Rank3Time					);
			BINDFUNC( Rank4Time					);
			BINDFUNC( Rank5Time					);
			BINDFUNC( AbilityStinkCloud			);
			BINDFUNC( AbilityElectricBurst		);
			BINDFUNC( AbilityQuillBurst			);
			BINDFUNC( AbilityFrenzy				);
			BINDFUNC( AbilityDigging			);
			BINDFUNC( AbilitySonarPulse			);
			BINDFUNC( AbilityPlague				);
            BINDFUNC( AbilityWebThrow			);
	    	BINDFUNC( AbilityAssassinate		);
		    BINDFUNC( AbilityFlash		        );

            BINDFUNC( DonatedRenew				);
			BINDFUNC( DonatedGather				);
			BINDFUNC( DamageBySBTower			);
			BINDFUNC( DamageByAATower			);
			BINDFUNC( PopulationMax				);
			BINDFUNC( CreatureUpgradeCount		);
			BINDFUNC( CreatureUpgradeTotalScrap );
			BINDFUNC( CreatureUpgradeTotalElectricity );
									
			BINDFUNC( ArmyMostEffective			);
			BINDFUNC( ArmyAvgCost				);
			BINDFUNC( ArmyAvgCostElectricity	);
			BINDFUNC( ArmyAvgCostScrap			);
			BINDFUNC( ArmyAvgRank				);

			BINDFUNC( DumpFramesToDisk );

		#undef BINDFUNC

		#define BINDFUNC_SPECIAL(f) \
			StoreLuaBindingObj( lc, LuaBinding::Bind( &lc, "Stats_"#f, StatLogging::Instance(), (int(StatLogging::*)(unsigned long, long)const)&StatLogging::f ) );
			BINDFUNC_SPECIAL( CreatureTotal			);
			BINDFUNC_SPECIAL( CreatureActive		);
			BINDFUNC_SPECIAL( CreatureLosses		);
			BINDFUNC_SPECIAL( CreatureKills			);
			BINDFUNC_SPECIAL( CreatureCreatureKills	);
		#undef BINDFUNC_SPECIAL

		#define BINDFUNC_SPECIAL_FLOAT(f) \
			StoreLuaBindingObj( lc, LuaBinding::Bind( &lc, "Stats_"#f, StatLogging::Instance(), (float(StatLogging::*)(unsigned long, long)const)&StatLogging::f ) );
			BINDFUNC_SPECIAL_FLOAT( CreatureSelfDmg		);
			BINDFUNC_SPECIAL_FLOAT( CreatureSelfDmgValue  );
		#undef BINDFUNC_SPECIAL_FLOAT

			return true;
		}

		virtual bool DeRegister( LuaConfig& lc )
		{
			ClearLuaBindingObj( lc );
			return true;
		}
	};

	///////////////////////////////////////////////////////////////////////////////
	// Declare a global static.  The constructor automaticall registers
	// itself with the LuaLibMan so that other libs have access to these functions
	LuaStatLoggingLib s_LuaStatLoggingLib;
}